﻿/*
   This source code is offered under the Microsoft Public License (Ms-PL) which is outlined as follows:

   Microsoft Public License (Ms-PL)
   This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.

   1. Definitions
   The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
   A "contribution" is the original software, or any additions or changes to the software.
   A "contributor" is any person that distributes its contribution under this license.
   "Licensed patents" are a contributor's patent claims that read directly on its contribution.

   2. Grant of Rights
   (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
   (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.

   3. Conditions and Limitations
   (A) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
   (B) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
   (C) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
   (D) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
   (E) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
 */
using System.Linq;
using System.Reflection;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System;
using System.Diagnostics.CodeAnalysis;

using NUnit.Framework;
using RenewalProjects.Libraries.Common.Testing.Interfaces;

namespace RenewalProjects.Libraries.Common.Testing.Attributes
{
	/// <summary>
	/// NUnit action to start up a WCF Service Host and close it afterwards.
	/// NOTE: Remember to set the types before the tests start.
	/// </summary>
	[AttributeUsage(AttributeTargets.Interface)]
	public sealed class UsesServiceHostsActionAttribute : TestActionAttribute
	{
		/// <summary>
		/// Starts up the service host before the test starts
		/// </summary>
		public override void BeforeTest(TestDetails testDetails)
		{
			IUsesServiceHosts fixtureInstance = testDetails.Fixture as IUsesServiceHosts;
			if(fixtureInstance != null)
			{
				ServiceHostMetadataAttribute metaData = GetServiceMetaData(testDetails);

				fixtureInstance.Address = metaData.Address;
				fixtureInstance.Binding = CreateInstanceBinding(metaData.BindingType);
				fixtureInstance.Host = new ServiceHost(metaData.Service);
				fixtureInstance.Host.AddServiceEndpoint(metaData.Contract,
					fixtureInstance.Binding,
					metaData.Address);
				fixtureInstance.Host.Open();
			}
		}

		/// <summary>
		/// Closes the service host after the test ended
		/// </summary>
		public override void AfterTest(TestDetails testDetails)
		{
			IUsesServiceHosts fixtureInstance = testDetails.Fixture as IUsesServiceHosts;
			if(fixtureInstance != null && fixtureInstance.Host != null)
			{
				fixtureInstance.Host.Close();
			}
		}

		/// <summary>
		/// Apply only to tests
		/// </summary>
		public override ActionTargets Targets
		{
			get { return ActionTargets.Test; }
		}

		private static Binding CreateInstanceBinding(Type bindingType)
		{
			return (Binding)Activator.CreateInstance(bindingType);
		}

		private static ServiceHostMetadataAttribute GetServiceMetaData(TestDetails testDetails)
		{
			ServiceHostMetadataAttribute result = GetMergedMetaData(testDetails);
			VerifyIfSpecified(result);
			DefaultIfNotSpecified(result);
			return result;
		}

		private static ServiceHostMetadataAttribute GetMergedMetaData(TestDetails testDetails)
		{
			ServiceHostMetadataAttribute result = new ServiceHostMetadataAttribute();
			ServiceHostMetadataAttribute fixtureMetaData = GetServiceHostMetaData(testDetails.Fixture.GetType());
			ServiceHostMetadataAttribute methodMetaData = GetServiceHostMetaData(testDetails.Method);

			CopyPropertiesIfSpecified(fixtureMetaData, result);
			CopyPropertiesIfSpecified(methodMetaData, result);

			return result;
		}

		[SuppressMessage("Microsoft.Usage", "CA2201:DoNotRaiseReservedExceptionTypes")]
		private static void VerifyIfSpecified(ServiceHostMetadataAttribute target)
		{
			if(target.Service == null)
			{
				throw new NullReferenceException("No Service type was specified");
			}

			if(target.Contract == null)
			{
				throw new NullReferenceException("No Contract type was specified");
			}
		}

		private static void DefaultIfNotSpecified(ServiceHostMetadataAttribute target)
		{
			if(target.Address == null)
			{
				target.Address = "net.pipe://localhost/test";
			}

			if(target.BindingType == null)
			{
				target.BindingType = typeof(NetNamedPipeBinding);
			}
		}

		private static void CopyPropertiesIfSpecified(ServiceHostMetadataAttribute source,
		                                              ServiceHostMetadataAttribute dest)
		{
			if(source != null)
			{
				if(source.Address != null)
				{
					dest.Address = source.Address;
				}

				if(source.BindingType != null)
				{
					dest.BindingType = source.BindingType;
				}

				if(source.Contract != null)
				{
					dest.Contract = source.Contract;
				}

				if(source.Service != null)
				{
					dest.Service = source.Service;
				}
			}
		}

		private static ServiceHostMetadataAttribute GetServiceHostMetaData(MemberInfo memberInfo)
		{
			return (ServiceHostMetadataAttribute)memberInfo.GetCustomAttributes(typeof(ServiceHostMetadataAttribute),
				false).FirstOrDefault();
		}
	}
}