﻿/*
   This source code is offered under the Microsoft Public License (Ms-PL) which is outlined as follows:

   Microsoft Public License (Ms-PL)
   This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.

   1. Definitions
   The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
   A "contribution" is the original software, or any additions or changes to the software.
   A "contributor" is any person that distributes its contribution under this license.
   "Licensed patents" are a contributor's patent claims that read directly on its contribution.

   2. Grant of Rights
   (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
   (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.

   3. Conditions and Limitations
   (A) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
   (B) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
   (C) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
   (D) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
   (E) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
 */
using System;
using System.Reflection;
using System.Security.Permissions;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Diagnostics.CodeAnalysis;

namespace RenewalProjects.Libraries.Common.Testing.Helpers
{
    /// <summary>
	/// This assists unit testers with throwing a bunch
	/// of threads against their piece of code to see
	/// what would happen. It's also has a fluent interface.
	/// </summary>
	public sealed class ThreadGroup
	{
		private readonly List<ThreadMetaData> _threads;

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="start">Delegate of what to do</param>
		/// <param name="amount">Amount of threads to make</param>
		public ThreadGroup(ThreadStart start, int amount)
		{
			if (start == null)
			{
				throw new ArgumentNullException("start");
			}

			if (amount < 1)
			{
				throw new ArgumentOutOfRangeException("amount");
			}

			_threads = new List<ThreadMetaData>(amount);

			for (int i = 0; i < amount; i++)
			{
				_threads.Add(new ThreadMetaData(start));
			}
		}

		/// <summary>
		/// Start up the threads and run them
		/// </summary>
		public ThreadGroup Start()
		{
			ReadyToken token = new ReadyToken();
			
			foreach (ThreadMetaData meta in _threads)
			{
				meta.StartThread(token);
			}
			
			token.SetReady();

			return this;
		}

		/// <summary>
		/// Wait for all the threads indefinitely
		/// </summary>
		public ThreadGroup WaitForThreads()
		{
			return WaitForThreads(TimeSpan.MaxValue);
		}

		/// <summary>
		/// Wait for all the threads but only for
		/// a specified time period
		/// </summary>
		/// <param name="timeout">
		/// Amount of time to elapse before
		/// Timeout exception is thrown
		/// </param>
		public ThreadGroup WaitForThreads(TimeSpan timeout)
		{
			DateTime prevTime = DateTime.Now;

			while (true)
			{
				if (DateTime.Now.Subtract(prevTime) >= timeout)
				{
					_threads.ForEach(a => a.AbortThread());
					throw new TimeoutException(
					          "Threads took longer than specified timeout of " + timeout);
				}

				int count = _threads.Where(p => p.IsDone()).Count();
				if (count == _threads.Count)
				{
					break;
				}
			}

			ThrowFirstCaughtException();

			return this;
		}

		private void ThrowFirstCaughtException()
		{
			Exception firstException = _threads.Select(s => s.CaughtException)
			                           .Where(p => p != null)
			                           .FirstOrDefault();

			if (firstException != null)
			{
				ThrowExceptionPreservingStack(firstException);
			}
		}

		[ReflectionPermission(SecurityAction.Demand)]
		private static void ThrowExceptionPreservingStack(Exception exception)
		{
			FieldInfo remoteStackTraceString =
			    typeof (Exception).GetField("_remoteStackTraceString",
			                                BindingFlags.Instance | BindingFlags.NonPublic);
			remoteStackTraceString.SetValue(exception, exception.StackTrace + Environment.NewLine);
			throw exception;
		}

		private class ReadyToken
		{
			private volatile bool _isReady;

			public void SetReady()
			{
				_isReady = true;
			}

			public void WaitTillReady()
			{
				while (!_isReady)
				{
					Thread.Sleep(10);
				}
			}
		}

		[SuppressMessage("Microsoft.Usage", "CA2201:DoNotRaiseReservedExceptionTypes")]
		private class ThreadMetaData
		{
			private readonly Thread _thread;
			private Exception _exception;

			[SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
			public ThreadMetaData(ThreadStart start)
			{
				_thread = new Thread(
				    (obj) =>
				    {
				        try
				        {
				        	((ReadyToken)obj).WaitTillReady();
				        	
				            start();
						}
				        catch (ThreadAbortException)
				        {
				            // Do nothing
						}
				        catch (Exception ex)
				        {
				            _exception = ex;
						}
					});
			}

			public void StartThread(ReadyToken token)
			{
				_thread.Start(token);
			}

			public void AbortThread()
			{
				_thread.Abort();
			}

			public bool IsDone()
			{
				return _thread.ThreadState == ThreadState.Stopped;
			}

			public Exception CaughtException
			{
				get { return _exception; }
			}
		}
	}
}
