﻿/*
   This source code is offered under the Microsoft Public License (Ms-PL) which is outlined as follows:

   Microsoft Public License (Ms-PL)
   This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.

   1. Definitions
   The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
   A "contribution" is the original software, or any additions or changes to the software.
   A "contributor" is any person that distributes its contribution under this license.
   "Licensed patents" are a contributor's patent claims that read directly on its contribution.

   2. Grant of Rights
   (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
   (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.

   3. Conditions and Limitations
   (A) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
   (B) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
   (C) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
   (D) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
   (E) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
 */

using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.IO;
using System.Text.RegularExpressions;
using System.Xml;
using System.Globalization;

namespace RenewalProjects.Applications.EncryptedConfigurationViewer
{
	/// <summary>
	/// Description of ConfigurationFileReader.
	/// </summary>
	public sealed class ConfigurationFileReader
	{
		private static readonly Regex _cleanupRegex = new Regex("[\\n\\r\\t]\\+", RegexOptions.IgnoreCase | RegexOptions.Compiled);

		private readonly Stream _configFileStream;
		private XmlDocument _doc;

		private const string PROTECTION_PROVIDER_ATTRIBUTE = "configProtectionProvider";
		private const string DATA_PROTECTION_CONFIGURATION_PROVIDER = "DataProtectionConfigurationProvider";
		private const string RSA_PROTECTION_CONFIGURATION_PROVIDER = "RsaProtectedConfigurationProvider";

		private IDictionary<string, ProtectedConfigurationProvider> _protectedConfigProvider;

		public ConfigurationFileReader(Stream configFileStream)
		{
			if (configFileStream == null)
			{
				throw new ArgumentNullException("configFileStream");
			}

			_configFileStream = configFileStream;
			_protectedConfigProvider = new Dictionary<string, ProtectedConfigurationProvider>
			{
				{ DATA_PROTECTION_CONFIGURATION_PROVIDER, new DpapiProtectedConfigurationProvider() },
				{ RSA_PROTECTION_CONFIGURATION_PROVIDER, GetRsaProtectedConfigurationProvider() }
			};
		}

		public void ReadFile()
		{
			_doc = new XmlDocument();
			_doc.Load(_configFileStream);

			NavigateThroughXmlNodeList(_doc.SelectNodes("/*"), _doc);
		}

		public string ProduceXml()
		{
			using (StringWriter stringWriter = new StringWriter(CultureInfo.InvariantCulture))
			{
				XmlTextWriter xmlWriter = new XmlTextWriter(stringWriter);

				xmlWriter.Formatting = Formatting.Indented;

				xmlWriter.Indentation = 2;

				xmlWriter.IndentChar = ' ';

				_doc.InnerXml = _cleanupRegex.Replace(_doc.InnerXml, "");

				_doc.WriteTo(xmlWriter);

				return stringWriter.ToString();
			}
		}

		private static RsaProtectedConfigurationProvider GetRsaProtectedConfigurationProvider()
		{
			RsaProtectedConfigurationProvider provider = new RsaProtectedConfigurationProvider();
			provider.Initialize("RsaProvider", new NameValueCollection { { "useMachineContainer", "true" } });

			return provider;
		}

		private void NavigateThroughXmlNodeList(XmlNodeList nodeList, XmlDocument doc)
		{
			foreach (XmlNode node in nodeList)
			{
				XmlElement element = node as XmlElement;
				if (element != null)
				{
					XmlAttribute protectProviderAttribute = element.Attributes[PROTECTION_PROVIDER_ATTRIBUTE];
					if (protectProviderAttribute != null)
					{
						ProtectedConfigurationProvider provider;
						if (_protectedConfigProvider.TryGetValue(protectProviderAttribute.Value, out provider))
						{
							XmlComment commentBegin = doc.CreateComment(" < Begin of encrypted part > ");
							XmlComment commentEnd = doc.CreateComment(" < End of encrypted part > ");
							
							element.RemoveAttribute(PROTECTION_PROVIDER_ATTRIBUTE);
							XmlNode decryptedNode = provider.Decrypt(element.FirstChild);
							
							element.ParentNode.InsertBefore(commentBegin, element);
							element.InnerXml = decryptedNode.InnerXml;
							element.ParentNode.InsertAfter(commentEnd, element);

							if (decryptedNode.Attributes != null)
							{
								foreach (XmlAttribute attr in decryptedNode.Attributes)
								{
									element.SetAttribute(attr.Name, attr.Value);
								}
							}
						}
					}

					NavigateThroughXmlNodeList(element.SelectNodes("*"), doc);
				}
			}
		}
	}
}
