﻿/*
   This source code is offered under the Microsoft Public License (Ms-PL) which is outlined as follows:

   Microsoft Public License (Ms-PL)
   This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.

   1. Definitions
   The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
   A "contribution" is the original software, or any additions or changes to the software.
   A "contributor" is any person that distributes its contribution under this license.
   "Licensed patents" are a contributor's patent claims that read directly on its contribution.

   2. Grant of Rights
   (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
   (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.

   3. Conditions and Limitations
   (A) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
   (B) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
   (C) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
   (D) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
   (E) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
 */

using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Windows.Forms;
using ICSharpCode.TextEditor;
using RenewalProjects.Applications.EncryptedConfigurationViewer.Properties;

namespace RenewalProjects.Applications.EncryptedConfigurationViewer
{
	/// <summary>
	/// Description of MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		protected MainForm()
		{
			//
			// The InitializeComponent() call is required for Windows Forms designer support.
			//
			InitializeComponent();

			SetupForm();
		}

		public MainForm(ParameterData parameters)
			: this()
		{
			OpenConfigurationIfSpecified(parameters);
		}

		[SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
		private void OpenConfigurationIfSpecified(ParameterData parameters)
		{
			try
			{
				if (!String.IsNullOrEmpty(parameters.ConfigurationPath))
				{
					OpenConfigurationFile(File.OpenRead(parameters.ConfigurationPath), parameters.ConfigurationPath);
				}
			}
			catch (Exception ex)
			{
				DisplayErrorMessage(ex.Message, "Error");
				xmlView.Text = "";
				Text = AppResource.TITLE;
			}
		}

		private void SetupForm()
		{
			xmlView.Text = "";
			xmlView.SetHighlighting("XML");
			xmlView.IsReadOnly = true;
			Text = AppResource.TITLE;
		}

		private void ExitToolStripMenuItemClick(object sender, EventArgs e)
		{
			Close();
		}

		[SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
		private void OpenToolStripMenuItemClick(object sender, EventArgs e)
		{
			try
			{
				OpenFileDialog openDialog = new OpenFileDialog();

				openDialog.Filter = ".NET Configuration Files | *.config";

				if (openDialog.ShowDialog() == DialogResult.OK)
				{
					OpenConfigurationFile(openDialog.OpenFile(), openDialog.FileName);
				}
			}
			catch (Exception ex)
			{
				DisplayErrorMessage(ex.Message, "Error");
				xmlView.Text = "";
				Text = AppResource.TITLE;
			}
		}

		private void AboutToolStripMenuItemClick(object sender, EventArgs e)
		{
			About about = new About();
			about.ShowDialog(this);
		}

		private void CopyToolStripMenuItemClick(object sender, EventArgs e)
		{
			CopyText();
		}

		private void SelectAllToolStripMenuItemClick(object sender, EventArgs e)
		{
			SelectAllText();
		}

		private void GotoToolStripMenuItemClick(object sender, EventArgs e)
		{
			GotoLine();
		}

		private void CopyText()
		{
			if (xmlView.Text.Length > 0)
			{
				string selectedText = xmlView.ActiveTextAreaControl.TextArea.SelectionManager.SelectedText;
				Clipboard.SetText(selectedText);
			}
		}

		private void GotoLine()
		{
			int totalLines = xmlView.Document.LineSegmentCollection.Count;
			GotoDialog gotoDlg = new GotoDialog(totalLines);
			gotoDlg.ShowDialog(this);
			if (gotoDlg.DialogResult == DialogResult.OK)
			{
				xmlView.ActiveTextAreaControl.JumpTo(gotoDlg.LineNumber - 1);
			}
		}

		private void SelectAllText()
		{
			ICSharpCode.TextEditor.Actions.SelectWholeDocument myselectAll = new ICSharpCode.TextEditor.Actions.SelectWholeDocument();
			myselectAll.Execute(xmlView.ActiveTextAreaControl.TextArea);
		}

		private void OpenConfigurationFile(Stream configStream, string configFilePath)
		{
			ConfigurationFileReader reader = new ConfigurationFileReader(configStream);
			reader.ReadFile();
			xmlView.Text = reader.ProduceXml();
			Text = AppResource.TITLE + " - " + Path.GetFileName(configFilePath);
		}
		
		private static void DisplayErrorMessage(string message, string title)
		{
			MessageBox.Show(message, title, MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1, MessageBoxOptions.DefaultDesktopOnly);
		}
	}
}
