﻿/*
   This source code is offered under the Microsoft Public License (Ms-PL) which is outlined as follows:

   Microsoft Public License (Ms-PL)
   This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.

   1. Definitions
   The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
   A "contribution" is the original software, or any additions or changes to the software.
   A "contributor" is any person that distributes its contribution under this license.
   "Licensed patents" are a contributor's patent claims that read directly on its contribution.

   2. Grant of Rights
   (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
   (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.

   3. Conditions and Limitations
   (A) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
   (B) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
   (C) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
   (D) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
   (E) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
 */
 
using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;

namespace RenewalProjects.Libraries.ParamGate
{
	/// <summary>
	/// A simple parameter list navigator.
	/// </summary>
	/// <remarks>
	/// It moves over each parameter as the MoveNext() is called
	/// but as it detects parameter values, it may navigate
	/// over all of them untill it reaches the next parameter key or
	/// the end of the parameter list. A parameter key is used
	/// to identify switches or the value paramters after it
	/// that may be associated with it.
	/// </remarks>
	public sealed class ParameterNavigator
	{
		private const string PARAM_PREFIX_CHAR = "-";

		private readonly string[] _parameters;
		private int _curPos = -1;

		/// <summary>
		/// Constructor
		/// </summary>
		public ParameterNavigator(string[] parameters)
		{
			if (parameters == null)
			{
				throw new ArgumentNullException("parameters");
			}

			_parameters = parameters;
		}

		/// <summary>
		/// Fetch next set of parameter key and/or values
		/// </summary>
		/// <returns>
		/// True if it managed to navigate, else False
		/// if it reeached the end.
		/// </returns>
		[SuppressMessage("Microsoft.Globalization", "CA1307:SpecifyStringComparison", MessageId = "System.String.StartsWith(System.String)")]
		public bool MoveNext()
		{
			Key = null;
			Values = null;

			int nextPos = _curPos + 1;
			if (nextPos < _parameters.Length)
			{
				_curPos = nextPos;
				nextPos = _curPos + 1;

				List<string> values = new List<string>();

				string curParam = _parameters[_curPos];
				if (curParam != null && curParam.StartsWith(PARAM_PREFIX_CHAR))
				{
					Key = curParam.Remove(0, 1);
				}
				else
				{
					values.Add(curParam);
				}

				string nextParam;
				while (nextPos < _parameters.Length
				       && ((nextParam = _parameters[nextPos]) == null
				           || !nextParam.StartsWith(PARAM_PREFIX_CHAR)))
				{
					_curPos = nextPos;
					nextPos = _curPos + 1;

					values.Add(nextParam);
				}

				Values = values;

				return true;
			}

			return false;
		}

		/// <summary>
		/// Gets the parameter Key
		/// </summary>
		public string Key { get; private set; }
		
		/// <summary>
		/// Gets the parameter Values
		/// </summary>
		public IList<string> Values { get; private set; }
	}
}
