﻿
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Threading;

using RenewalProjects.Libraries.Wcf.EasyProxy.Pools;
using WcfLibTest.Tests.Helpers;

namespace WcfLibTest.Tests
{
	public class ListPoolTester : TestBase
	{
		#region TestBase

		public override string GetDescription()
		{
			return "Test ListPool logic and concurrency";
		}

		protected override void TestBegin()
		{
			TestItem.ResetItems();
		}

		#endregion

		#region Tests

		public void TestListPoolWithOneThreadInUse()
		{
			ListPool<TestItem> pool = new ListPool<TestItem>();
			pool.ItemFactory = () => new TestItem();

			ThreadGroup th =
			    new ThreadGroup(
			        () =>
			        {
			            TestItem item = pool.GetAvailableItem();
					}, 1)
			    .Start()
			    .WaitForThreads();

			PrintItemsInUse();
			TestItem.PrintDrainCount();
		}

		public void TestListPoolWithFullPoolAmountThreads()
		{
			ListPool<TestItem> pool = new ListPool<TestItem>();
			pool.ItemFactory = () => new TestItem();

			ThreadGroup th =
			    new ThreadGroup(
			        () =>
			        {
			            TestItem item = pool.GetAvailableItem();
			            if (item != null)
			            {
			                item.DoSomething();
						}
			            else
			            {
			                Console.WriteLine("Item is null");
						}
					}, 10)
			    .Start()
			    .WaitForThreads();

			PrintItemsInUse();
			TestItem.PrintDrainCount();
		}

		public void TestListPoolWithContinuousThreads()
		{
			ListPool<TestItem> pool = new ListPool<TestItem>();
			pool.ItemFactory = () => new TestItem();

			for (int i = 0; i < 10; i++)
			{
				ThreadGroup th =
				    new ThreadGroup(
				        () =>
				        {
				            TestItem item = pool.GetAvailableItem();
				            if (item != null)
				            {
				                item.DoSomething();
							}
				            else
				            {
				                Console.WriteLine("Item is null");
							}
						}, 30)
				    .Start()
				    .WaitForThreads();
			}

			PrintItemsInUse();
			TestItem.PrintDrainCount();
		}

		public void TestListPoolWithContinuousThreadsAndExpiryTestItems()
		{
			ListPool<ExpiryTestItem> pool = new ListPool<ExpiryTestItem>(15);
			pool.ItemFactory = () => new ExpiryTestItem();

			for (int i = 0; i < 10; i++)
			{
				ThreadGroup th =
				    new ThreadGroup(
				        () =>
				        {
				            TestItem item = pool.GetAvailableItem();
				            if (item != null)
				            {
				                item.DoSomething();
							}
				            else
				            {
				                Console.WriteLine("Item is null");
							}
						}, 15)
				    .Start()
				    .WaitForThreads();
				Console.WriteLine("Completed iteration {0}", i + 1);
			}

			PrintItemsInUse();
			TestItem.PrintDrainCount();
		}

		#endregion

		#region Test Helpers

		private static void PrintItemsInUse()
		{
			int inUseCount = TestItem.GetItems().Select(s => s.InUse).Where(p => p).Count();
			int total = TestItem.GetItems().Count();

			Console.WriteLine("{0} / {1} is still in use", inUseCount, total);
		}

		private class TestItem : IPoolItem
		{
			private static ConcurrentBag<TestItem> _items;

			private static volatile int _drainCount;
			private volatile bool _inUse;

			public TestItem()
			{
				_items.Add(this);
			}

			public static IEnumerable<TestItem> GetItems()
			{
				return _items;
			}

			public static void ResetItems()
			{
				_items = new ConcurrentBag<TestItem>();
				_drainCount = 0;
			}

			public virtual void DoSomething()
			{
				InUse = false;
			}

			public bool InUse
			{
				get
				{
					return _inUse;
				}
				set
				{
					_inUse = value;
				}
			}

			public virtual bool CheckItemExpiry()
			{
				return false;
			}

			public void DrainItem()
			{
				_drainCount++;
			}

			public static void PrintDrainCount()
			{
				Console.WriteLine("Drain Count : {0}", _drainCount);
			}
		}

		private class ExpiryTestItem : TestItem
		{
			private static Random _random = new Random(14687923);
			private volatile int _count;
			private DateTime _prevTime = DateTime.Now;

			public override void DoSomething()
			{
				if (_random.Next(0, 100) >= 50)
				{
					InUse = false;
					Console.WriteLine("InUse = false");
				}
			}

			public override bool CheckItemExpiry()
			{
				_count++;
				return _count == 2
				       || (DateTime.Now.Subtract(_prevTime).TotalMilliseconds > 5);
			}
		}

		#endregion
	}
}
