﻿/*
   This source code is offered under the Microsoft Public License (Ms-PL) which is outlined as follows:

   Microsoft Public License (Ms-PL)
   This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.

   1. Definitions
   The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
   A "contribution" is the original software, or any additions or changes to the software.
   A "contributor" is any person that distributes its contribution under this license.
   "Licensed patents" are a contributor's patent claims that read directly on its contribution.

   2. Grant of Rights
   (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
   (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.

   3. Conditions and Limitations
   (A) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
   (B) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
   (C) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
   (D) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
   (E) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
 */
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System;

using RenewalProjects.Libraries.Wcf.EasyProxy.ChannelAllocators.Settings;
using RenewalProjects.Libraries.Wcf.EasyProxy.ChannelHolders;
using RenewalProjects.Libraries.Wcf.EasyProxy.Pools;

namespace RenewalProjects.Libraries.Wcf.EasyProxy.ChannelAllocators.Factories
{
	/// <summary>
	/// This is an internal factory for creating pooled channel resource providers.
	/// This helps with code simplification and making it unit testible.
	/// </summary>
	internal static class PooledChannelResourceProviderFactory<TContract>
	{
		/// <summary>
		/// Static constructor
		/// </summary>
		static PooledChannelResourceProviderFactory()
		{
			ResetFactoryMethods();
		}
		
		/// <summary>
		/// Make sure the default factory methods are applied
		/// </summary>
		public static void ResetFactoryMethods()
		{
		    ReadyChannelHolderFactory = (s) => new ResourceListPool<ReadyChannelHolder<TContract>>(
			        s.ReadyChannelMax);
			StandbyChannelHolderFactory = (s) => new ResourceListPool<StandbyChannelHolder<TContract>>(
			        s.StandbyChannelMax);
		}

		/// <summary>
		/// Factory method for creating a resource list pool for a ready channel holder resource item
		/// </summary>
		public static Func<PoolChannelAllocatorSettings,
		                   IResourceItemProvider<ReadyChannelHolder<TContract>>> ReadyChannelHolderFactory { get; set; }

		/// <summary>
		/// Factory method for creating a resource list pool for a standby channel holder resource item
		/// </summary>
		public static Func<PoolChannelAllocatorSettings,
		                   IResourceItemProvider<StandbyChannelHolder<TContract>>> StandbyChannelHolderFactory { get; set; }

		/// <summary>
		/// Produces a Ready resource item provider
		/// </summary>
		/// <param name="channelFactory">WCF Channel factory</param>
		/// <param name="serviceEndpoint">Service endpoint</param>
		/// <param name="settings">Pooled channel allocator settings</param>
		public static IResourceItemProvider<ReadyChannelHolder<TContract>> SupplyReadyResourceItemProvider(
		        IChannelFactory<TContract> channelFactory,
		        ServiceEndpoint serviceEndpoint,
		        PoolChannelAllocatorSettings settings)
		{
			IResourceItemProvider<ReadyChannelHolder<TContract>> readyPool = null;

			if (settings.ReadyChannelMax > 0)
			{
				readyPool = ReadyChannelHolderFactory(settings);
				readyPool.ResourceRefillThreshold = settings.ReadyChannelRefillThreshold;
			}

			if (readyPool != null)
			{
				readyPool.ResourceItemFactory = () => new ReadyChannelHolder<TContract>(
				        channelFactory.CreateChannel(serviceEndpoint.Address, null),
				        settings.MaxChannelOpenTime);
			}

			return readyPool;
		}

		/// <summary>
		/// Produces a Standby resource item provider
		/// </summary>
		/// <param name="channelFactory">WCF Channel factory</param>
		/// <param name="serviceEndpoint">Service endpoint</param>
		/// <param name="settings">Pooled channel allocator settings</param>
		public static IResourceItemProvider<StandbyChannelHolder<TContract>> SupplyStandbyResourceItemProvider(
		        IChannelFactory<TContract> channelFactory,
		        ServiceEndpoint serviceEndpoint,
		        PoolChannelAllocatorSettings settings)
		{
			IResourceItemProvider<StandbyChannelHolder<TContract>> standbyPool = null;

			if (settings.StandbyChannelMax > 0)
			{
				standbyPool = StandbyChannelHolderFactory(settings);
				standbyPool.ResourceRefillThreshold = settings.StandbyChannelRefillThreshold;
			}

			if (standbyPool != null)
			{
				standbyPool.ResourceItemFactory = () => new StandbyChannelHolder<TContract>(
				        channelFactory.CreateChannel(serviceEndpoint.Address, null),
				        settings.MaxChannelOpenTime);
			}

			return standbyPool;
		}
	}
}