﻿/*
   This source code is offered under the Microsoft Public License (Ms-PL) which is outlined as follows:

   Microsoft Public License (Ms-PL)
   This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.

   1. Definitions
   The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
   A "contribution" is the original software, or any additions or changes to the software.
   A "contributor" is any person that distributes its contribution under this license.
   "Licensed patents" are a contributor's patent claims that read directly on its contribution.

   2. Grant of Rights
   (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
   (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.

   3. Conditions and Limitations
   (A) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
   (B) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
   (C) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
   (D) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
   (E) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
 */
using System.Collections.Generic;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.ServiceModel;
using System.Threading;
using System;

using RenewalProjects.Libraries.Wcf.EasyProxy.ChannelAllocators.Factories;
using RenewalProjects.Libraries.Wcf.EasyProxy.ChannelAllocators.Settings;
using RenewalProjects.Libraries.Wcf.EasyProxy.ChannelHolders;
using RenewalProjects.Libraries.Wcf.EasyProxy.Pools;

namespace RenewalProjects.Libraries.Wcf.EasyProxy.ChannelAllocators
{
	/// <summary>
	/// This is a channel pool allocator that will ensure that
	/// enough channels are available to be (re)used and recycled upon completion.
	/// If not enough channels are available, it will provide transient channels.
	/// </summary>
	public sealed class PooledChannelAllocator<TContract>: ChannelAllocatorBase<TContract>
	{
		private readonly IResourceItemProvider<ReadyChannelHolder<TContract>> _readyPool;
		private readonly IResourceItemProvider<StandbyChannelHolder<TContract>> _standbyPool;
		private readonly PoolChannelAllocatorSettings _settings;

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="channelFactory">Channel Factory</param>
		/// <param name="serviceEndpoint">Service Endpoint</param>
		public PooledChannelAllocator(IChannelFactory<TContract> channelFactory, ServiceEndpoint serviceEndpoint)
			: this(channelFactory, serviceEndpoint, new PoolChannelAllocatorSettings())
		{
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="channelFactory">Channel Factory</param>
		/// <param name="serviceEndpoint">Service Endpoint</param>
		/// <param name="settings">Pool channel allocator settings</param>
		public PooledChannelAllocator(IChannelFactory<TContract> channelFactory,
		                              ServiceEndpoint serviceEndpoint,
		                              PoolChannelAllocatorSettings settings)
			: base(channelFactory, serviceEndpoint)
		{
			_settings = settings;
			_settings.ValidateProperties();

			_readyPool = PooledChannelResourceProviderFactory<TContract>.SupplyReadyResourceItemProvider(
			        channelFactory,
			        serviceEndpoint,
			        settings);
			_standbyPool = PooledChannelResourceProviderFactory<TContract>.SupplyStandbyResourceItemProvider(
			        channelFactory,
			        serviceEndpoint,
			        settings);
		}

		/// <summary>
		/// Manually refresh the channel pools.
		/// Just make sure that the services are available
		/// at the time of calling this, especially if you
		/// have ready-channels configured.
		/// </summary>
		public void RefreshChannelPools()
		{
			if (_readyPool != null)
			{
				_readyPool.RefreshResources();
			}

			if (_standbyPool != null)
			{
				_standbyPool.RefreshResources();
			}
		}

		/// <summary>
		/// Produce the necessary channel to be consumed
		/// by the proxy.
		/// </summary>
		public override ChannelHolderBase<TContract> GetAvailableChannel()
		{
			ChannelHolderBase<TContract> item;

			if (_readyPool != null)
			{
				item = _readyPool.GetAvailableResource();
				if (item != null)
				{
					return item;
				}
			}

			if (_standbyPool != null)
			{
				item = _standbyPool.GetAvailableResource();
				if (item != null)
				{
					return item;
				}
			}

			return new TransientChannelHolder<TContract>(
			               _channelFactory.CreateChannel(_serviceEndpoint.Address, null));
		}
	}
}