﻿/*
   This source code is offered under the Microsoft Public License (Ms-PL) which is outlined as follows:

   Microsoft Public License (Ms-PL)
   This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.

   1. Definitions
   The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
   A "contribution" is the original software, or any additions or changes to the software.
   A "contributor" is any person that distributes its contribution under this license.
   "Licensed patents" are a contributor's patent claims that read directly on its contribution.

   2. Grant of Rights
   (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
   (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.

   3. Conditions and Limitations
   (A) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
   (B) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
   (C) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
   (D) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
   (E) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
 */
using System;
using System.ServiceModel;
using RenewalProjects.Libraries.Wcf.EasyProxy.Pools;

namespace RenewalProjects.Libraries.Wcf.EasyProxy.ChannelHolders
{
	/// <summary>
	/// A channel that is usable but only opens upon usage.
	/// </summary>
	public sealed class StandbyChannelHolder<TContract> : ChannelHolderBase<TContract>, IResourceItem
	{
		private volatile bool _inUse;
		private readonly TimeSpan? _maxTime;
		private readonly DateTime _firstTime = DateTime.Now;

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="channel">Communication Object Channel</param>
		/// <param name="maxTime">
		/// Maximum time a channel is allowed to be open.
		/// Null is indefinitely.
		/// </param>
		public StandbyChannelHolder(TContract channel, TimeSpan ? maxTime)
			: base(channel)
		{
			_maxTime = maxTime;
		}

		/// <summary>
		/// Gets or sets the usage flag for Item Pool
		/// </summary>
		public bool InUse
		{
			get { return _inUse; }
			set { _inUse = value; }
		}

		/// <summary>
		/// The Service access consumer signals that it
		/// is done with the channel.
		/// </summary>
		public override void EndCall()
		{
			InUse = false;
		}

		/// <summary>
		/// The Service access consumer signals that it
		/// has encountered an error with the channel.
		/// </summary>
		public override void AbortCall()
		{
			base._commObject.Abort();
		}

		/// <summary>
		/// Determine if the item is still usable.
		/// </summary>
		public bool CheckItemExpiry()
		{
			return base._commObject.State == CommunicationState.Faulted
			       || base._commObject.State == CommunicationState.Closed
			       || base._commObject.State == CommunicationState.Closing
			       || TimeExpired();
		}

		/// <summary>
		/// Cleans up the channel
		/// </summary>
		public void DrainItem()
		{
			if (base._commObject.State == CommunicationState.Opened)
			{
				base._commObject.Close();
			}

			if (base._commObject.State == CommunicationState.Faulted)
			{
				base._commObject.Abort();
			}
		}

		private bool TimeExpired()
		{
			return _maxTime != null && DateTime.Now.Subtract(_firstTime) > _maxTime;
		}
	}
}
