﻿/*
   This source code is offered under the Microsoft Public License (Ms-PL) which is outlined as follows:

   Microsoft Public License (Ms-PL)
   This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.

   1. Definitions
   The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
   A "contribution" is the original software, or any additions or changes to the software.
   A "contributor" is any person that distributes its contribution under this license.
   "Licensed patents" are a contributor's patent claims that read directly on its contribution.

   2. Grant of Rights
   (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
   (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.

   3. Conditions and Limitations
   (A) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
   (B) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
   (C) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
   (D) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
   (E) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
 */
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.ServiceModel;
using System;

using RenewalProjects.Libraries.Wcf.EasyProxy.CallBehaviors;
using RenewalProjects.Libraries.Wcf.EasyProxy.ChannelAllocators;

namespace RenewalProjects.Libraries.Wcf.EasyProxy.ProxyFactories.Configuration
{
    #region Delegates
    
	/// <summary>
	/// Delegate for generating a channel factory
	/// </summary>
	public delegate ChannelFactoryResult<TContract> ChannelFactoryDelegate<TContract>();

	/// <summary>
	/// Delegate for generating a channel allocator
	/// </summary>
	public delegate ChannelAllocatorBase<TContract> ChannelAllocatorDelegate<TContract>(
	        IChannelFactory<TContract> channelFactory, ServiceEndpoint endpoint);

	/// <summary>
	/// Delegate for generating a call behavior
	/// </summary>
	public delegate CallBehaviorBase<TContract> CallBehaviorDelegate<TContract>(
	        ChannelAllocatorBase<TContract> channelAllocatorFactory);
	
	#endregion
	
	#region Easy Proxy Factory Configuration

	/// <summary>
	/// Configuration instance for a specific contract for creating various
	/// aspects of the EaxyProxy proxies.
	/// </summary>
	public sealed class EasyProxyFactoryConfiguration<TContract>
	{
		private ChannelFactoryDelegate<TContract> _channelFactoryFactory;
		private ChannelAllocatorDelegate<TContract> _channelAllocatorFactory;
		private CallBehaviorDelegate<TContract> _callBehaviorFactory;

		/// <summary>
		/// Default constructor
		/// </summary>
		public EasyProxyFactoryConfiguration()
		{
			ResetToDefaults();
		}

        /// <summary>
        /// Sets the entire configuration for the given contract back to default
        /// </summary>
        public void ResetToDefaults()
        {
            _channelFactoryFactory = () => new ChannelFactoryResult<TContract>(new ChannelFactory<TContract>());
            _channelAllocatorFactory = (cf, e) => new TransientChannelAllocator<TContract>(cf, e);
            _callBehaviorFactory = (ca) => new DefaultCallBehavior<TContract>(ca);
        }

		/// <summary>
		/// Gets or sets the Channel factory's factory
		/// </summary>
		public ChannelFactoryDelegate<TContract> ChannelFactoryFactory
		{
			get { return _channelFactoryFactory; }
			set
			{
				if(value == null)
				{
					throw new NullReferenceException("Cannot set channel factory to be null");
				}

				_channelFactoryFactory = value;
			}
		}

		/// <summary>
		/// Gets or sets the Channel allocator's factory
		/// </summary>
		public ChannelAllocatorDelegate<TContract> ChannelAllocatorFactory
		{
			get { return _channelAllocatorFactory; }
			set
			{
				if(value == null)
				{
					throw new NullReferenceException("Cannot set channel allocator factory to be null");
				}

				_channelAllocatorFactory = value;
			}
		}
		
		/// <summary>
		/// Gets or sets the Call behavior's factory
		/// </summary>
		public CallBehaviorDelegate<TContract> CallBehaviorFactory
		{
		    get { return _callBehaviorFactory; }
			set
			{
				if(value == null)
				{
					throw new NullReferenceException("Cannot set call behavior factory to be null");
				}

				_callBehaviorFactory = value;
			}
		}
	}
	
	#endregion
	
	#region Helper Code
	
	/// <summary>
	/// A way to obtain both a Channel Factory and the Service Endpoint
	/// </summary>
	public sealed class ChannelFactoryResult<TContract>
	{
		private readonly IChannelFactory<TContract> _channelFactory;
		private readonly ServiceEndpoint _endpoint;

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="channelFactory">Channel factory</param>
		/// <param name="endpoint">Channel factory's service endpoint</param>
		public ChannelFactoryResult(IChannelFactory<TContract> channelFactory, ServiceEndpoint endpoint)
		{
			_channelFactory = channelFactory;
			_endpoint = endpoint;
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="channelFactory">Channel factory</param>
		public ChannelFactoryResult(ChannelFactory<TContract> channelFactory)
		{
			_channelFactory = channelFactory;
			_endpoint = channelFactory.Endpoint;
		}

		/// <summary>
		/// Gets a channel factory
		/// </summary>
		public IChannelFactory<TContract> ChannelFactory
		{
			get { return _channelFactory; }
		}

		/// <summary>
		/// Gets a service endpoint
		/// </summary>
		public ServiceEndpoint Endpoint
		{
			get { return _endpoint; }
		}
	}
	
	#endregion
}