﻿/*
   This source code is offered under the Microsoft Public License (Ms-PL) which is outlined as follows:

   Microsoft Public License (Ms-PL)
   This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.

   1. Definitions
   The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
   A "contribution" is the original software, or any additions or changes to the software.
   A "contributor" is any person that distributes its contribution under this license.
   "Licensed patents" are a contributor's patent claims that read directly on its contribution.

   2. Grant of Rights
   (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
   (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.

   3. Conditions and Limitations
   (A) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
   (B) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
   (C) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
   (D) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
   (E) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
 */
using System.Collections.Generic;
using System.ServiceModel;
using System;

using LinFu.DynamicProxy;
using RenewalProjects.Libraries.Wcf.EasyProxy.CallBehaviors;
using RenewalProjects.Libraries.Wcf.EasyProxy.ChannelAllocators;
using RenewalProjects.Libraries.Wcf.EasyProxy.ProxyFactories.Configuration;
using RenewalProjects.Libraries.Wcf.EasyProxy.ProxyFactories.Interceptors;

namespace RenewalProjects.Libraries.Wcf.EasyProxy.ProxyFactories
{
	/// <summary>
	/// Easy proxy is a dynamic proxy factory for WCF proxies which
	/// takes in a Channel Factory and produces WCF channels which are
	/// managed on the caller's behalf.
	/// The Easy proxy user only needs to create one proxy and it
	/// can be reused multiple times (even on different threads).
	/// </summary>
	public static class EasyProxyFactory
	{
		private static readonly ProxyFactory _proxyFactory = new ProxyFactory();
		private static readonly IDictionary<Type, object> _configurations = new Dictionary<Type, object>();

		/// <summary>
		/// Get the configuration for a given contract (which can be used to configure
		/// the factories for that given contract).
		/// </summary>
		public static EasyProxyFactoryConfiguration<TContract> GetConfiguration<TContract>()
		{
			object item;
			EasyProxyFactoryConfiguration<TContract> config;
			if(!_configurations.TryGetValue(typeof(TContract), out item))
			{
				config = new EasyProxyFactoryConfiguration<TContract>();
				_configurations.Add(typeof(TContract), config);
			}
			else
			{
				config = (EasyProxyFactoryConfiguration<TContract>)item;
			}

			return config;
		}

        /// <summary>
        /// Creates a proxy by which the default (or global) configuration is used
        /// to define its behavior.
        /// </summary>
        public static TContract CreateProxy<TContract>()
        {
            return _proxyFactory.CreateProxy<TContract>(new DefaultProxyInterceptor<TContract>(CreateCallBehavior<TContract>()));
        }

		/// <summary>
		/// Creates a proxy by which the user specifies the call behavior to use.
		/// </summary>
		/// <param name="callBehavior">The desired call behavior</param>
		public static TContract CreateProxy<TContract>(CallBehaviorBase<TContract> callBehavior)
		{
			return _proxyFactory.CreateProxy<TContract>(new DefaultProxyInterceptor<TContract>(callBehavior));
		}

		private static ChannelAllocatorBase<TContract> CreateChannelAllocator<TContract>()
		{
			EasyProxyFactoryConfiguration<TContract> config = GetConfiguration<TContract>();
			ChannelFactoryResult<TContract> channelFactoryResult = config.ChannelFactoryFactory();
			return config.ChannelAllocatorFactory(channelFactoryResult.ChannelFactory, channelFactoryResult.Endpoint);
		}
		
		private static CallBehaviorBase<TContract> CreateCallBehavior<TContract>()
		{
		    EasyProxyFactoryConfiguration<TContract> config = GetConfiguration<TContract>();
		    ChannelAllocatorBase<TContract> channelAllocator = CreateChannelAllocator<TContract>();
		    return config.CallBehaviorFactory(channelAllocator);
		}
	}
}